/*
GLDM - Graphical Liberty Display Manager

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "unixauthprovider.h"

#ifdef HAVE_SHADOW
#include <shadow.h>
#endif
#include <sys/types.h>
#include <pwd.h>
#include <string.h>
#include <fstream>
#include <sstream>
#include <unistd.h>


static const char etc_passwd[] = "/etc/passwd";

UnixAuthProvider::UnixAuthProvider()
{
    loadUsers(1000, 32767, etc_passwd);
}


bool UnixAuthProvider::checkOwnAuth(const std::string& login, const std::string& password)
{
    char *encrypted, *correct;
    struct passwd *pw;

    pw = getpwnam(login.c_str());

    endpwent();
    if(pw == 0)
        return false;

#ifdef HAVE_SHADOW
    struct spwd *sp = getspnam(pw->pw_name);
    endspent();
    if(sp)
        correct = sp->sp_pwdp;
    else
#endif        // HAVE_SHADOW
        correct = pw->pw_passwd;

    if(correct == 0 || correct[0] == '\0')
        return true;

    encrypted = crypt(password.c_str(), correct);
    return ((strcmp(encrypted, correct) == 0) ? true : false);

}


/** \todo document this
  * \todo incorporate into class?
  */

static std::string getPasswdSToken(std::ifstream &passf, char term = ':') {
	if (passf.get() == term)
		return std::string();
	std::stringbuf buf;
	passf.unget();
	passf.get(buf, term);
	passf.get();
	return buf.str();
}

/** \todo document this
  * \todo incorporate into class
  */
static int getPasswdIToken(std::ifstream &passf) {
	int i;
	passf >> i;
	passf.get();
	return i;
}



void UnixAuthProvider::loadUsers(int minUID, int maxUID, const char* passwd)
{
    // TODO examine if we should use getpwnam() for reading (some) user data
	User user;
	std::ifstream passf;

	passf.open(passwd);
	while (passf.good()) {
		user.username = getPasswdSToken(passf);
		if (user.username.empty())
			break;
		getPasswdSToken(passf);
		user.uid = getPasswdIToken(passf);
		user.gid = getPasswdIToken(passf);
		user.info = getPasswdSToken(passf); user.info = user.info.substr(0,user.info.find(','));
		user.home = getPasswdSToken(passf);
		user.shell = getPasswdSToken(passf, '\n');

		//user.dump();
		if (user.uid >= minUID && user.uid <= maxUID)
			mUsers.push_back(user);
	}
	passf.close();
}


std::vector<std::string> UnixAuthProvider::getOwnLoginListUsers()
{
    std::vector<std::string> z;
    for(std::vector<User>::iterator it = mUsers.begin(); it != mUsers.end(); it++)
    {
        z.push_back(it->username);
    }
    return z;
}


std::string UnixAuthProvider::getOwnDisplayName(std::string username)
{
    std::vector<std::string> z;
    for(std::vector<User>::iterator it = mUsers.begin(); it != mUsers.end(); it++)
    {
        if (it->username == username)
        {
            return it->info;
        }
    }
    return "";
}

std::string UnixAuthProvider::getOwnHomeDir(std::string username)
{
    std::vector<std::string> z;
    for(std::vector<User>::iterator it = mUsers.begin(); it != mUsers.end(); it++)
    {
        if (it->username == username)
        {
            return it->home;
        }
    }
    return "";
}
